import {
  Body,
  Controller,
  Delete,
  Get,
  Patch,
  Path,
  Post,
  Request,
  Route,
  Security,
  Tags,
} from "tsoa";
import { AppDataSource } from "../database/data-source";
import HttpSuccess from "../interfaces/http-success";
import HttpStatus from "../interfaces/http-status";
import HttpError from "../interfaces/http-error";
import { ProfileDisciplineHistory } from "../entities/ProfileDisciplineHistory";
import { RequestWithUser } from "../middlewares/user";
import {
  CreateProfileEmployeeDiscipline,
  ProfileDiscipline,
  UpdateProfileDiscipline,
} from "../entities/ProfileDiscipline";
import { ProfileEmployee } from "../entities/ProfileEmployee";
import permission from "../interfaces/permission";
import { setLogDataDiff } from "../interfaces/utils";
@Route("api/v1/org/profile-employee/discipline")
@Tags("ProfileDisciplineEmployee")
@Security("bearerAuth")
export class ProfileDisciplineEmployeeController extends Controller {
  private profileRepository = AppDataSource.getRepository(ProfileEmployee);
  private disciplineRepository = AppDataSource.getRepository(ProfileDiscipline);
  private disciplineHistoryRepository = AppDataSource.getRepository(ProfileDisciplineHistory);

  @Get("user")
  public async getDisciplineUser(@Request() request: { user: Record<string, any> }) {
    const profile = await this.profileRepository.findOneBy({ keycloak: request.user.sub });
    if (!profile) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "ไม่พบ profile ดังกล่าว");
    }
    const lists = await this.disciplineRepository.find({
      where: { profileEmployeeId: profile.id, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(lists);
  }

  @Get("{profileId}")
  public async getDiscipline(@Path() profileId: string, @Request() req: RequestWithUser) {
    let _workflow = await new permission().Workflow(req, profileId, "SYS_REGISTRY_EMP");
    if (_workflow == false)
      await new permission().PermissionOrgUserGet(req, "SYS_REGISTRY_EMP", profileId);
    const lists = await this.disciplineRepository.find({
      where: { profileEmployeeId: profileId, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(lists);
  }

  @Get("admin/{profileId}")
  public async getDisciplineAdmin(@Path() profileId: string, @Request() req: RequestWithUser) {
    let _workflow = await new permission().Workflow(req, profileId, "SYS_WAGE");
    if (_workflow == false) await new permission().PermissionGet(req, "SYS_WAGE");
    const lists = await this.disciplineRepository.find({
      where: { profileEmployeeId: profileId, isDeleted: false },
      order: { createdAt: "ASC" },
    });
    return new HttpSuccess(lists);
  }

  @Get("admin/history/{disciplineId}")
  public async disciplineAdminHistory(
    @Path() disciplineId: string,
    @Request() req: RequestWithUser,
  ) {
    const _record = await this.disciplineRepository.findOneBy({ id: disciplineId });
    if (_record) {
      let _workflow = await new permission().Workflow(req, disciplineId, "SYS_REGISTRY_EMP");
      if (_workflow == false)
        await new permission().PermissionOrgUserGet(
          req,
          "SYS_REGISTRY_EMP",
          _record.profileEmployeeId,
        );
    }
    const record = await this.disciplineHistoryRepository.find({
      where: { profileDisciplineId: disciplineId },
      order: { createdAt: "DESC" },
    });
    return new HttpSuccess(record);
  }

  @Get("history/{disciplineId}")
  public async disciplineHistory(@Path() disciplineId: string) {
    const record = await this.disciplineHistoryRepository.find({
      where: { profileDisciplineId: disciplineId },
      order: { createdAt: "DESC" },
    });
    return new HttpSuccess(record);
  }

  @Post()
  public async newDiscipline(
    @Request() req: RequestWithUser,
    @Body() body: CreateProfileEmployeeDiscipline,
  ) {
    if (!body.profileEmployeeId) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "กรุณากรอก profileId");
    }

    const profile = await this.profileRepository.findOneBy({ id: body.profileEmployeeId });
    if (!profile) {
      throw new HttpError(HttpStatus.BAD_REQUEST, "ไม่พบ profile ดังกล่าว");
    }
    await new permission().PermissionOrgUserUpdate(req, "SYS_REGISTRY_EMP", profile.id);
    const before = null;
    const data = new ProfileDiscipline();

    const meta = {
      createdUserId: req.user.sub,
      createdFullName: req.user.name,
      lastUpdateUserId: req.user.sub,
      lastUpdateFullName: req.user.name,
      createdAt: new Date(),
      lastUpdatedAt: new Date(),
    };

    Object.assign(data, { ...body, ...meta });
    const history = new ProfileDisciplineHistory();
    Object.assign(history, { ...data, id: undefined });

    await this.disciplineRepository.save(data, { data: req });
    setLogDataDiff(req, { before, after: data });
    history.profileDisciplineId = data.id;
    await this.disciplineHistoryRepository.save(history, { data: req });
    //setLogDataDiff(req, { before, after: history });
    return new HttpSuccess(data.id);
  }

  @Patch("{disciplineId}")
  public async editDiscipline(
    @Request() req: RequestWithUser,
    @Body() body: UpdateProfileDiscipline,
    @Path() disciplineId: string,
  ) {
    const record = await this.disciplineRepository.findOneBy({ id: disciplineId });
    if (!record) throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");
    await new permission().PermissionOrgUserUpdate(
      req,
      "SYS_REGISTRY_EMP",
      record.profileEmployeeId,
    );
    const before = structuredClone(record);
    // const before_null = null;
    const history = new ProfileDisciplineHistory();

    Object.assign(record, body);
    Object.assign(history, { ...record, id: undefined });

    history.profileDisciplineId = disciplineId;
    record.lastUpdateUserId = req.user.sub;
    record.lastUpdateFullName = req.user.name;
    record.lastUpdatedAt = new Date();
    history.lastUpdateUserId = req.user.sub;
    history.lastUpdateFullName = req.user.name;
    history.createdUserId = req.user.sub;
    history.createdFullName = req.user.name;
    history.createdAt = new Date();
    history.lastUpdatedAt = new Date();

    this.disciplineRepository.save(record, { data: req });
    setLogDataDiff(req, { before, after: record });
    if (!(Object.keys(body).length === 1 && body.isUpload)) {
      this.disciplineHistoryRepository.save(history, { data: req });
      // setLogDataDiff(req, { before, after: history });
    }

    return new HttpSuccess();
  }

  /**
   * API ลบข้อมูลวินัย
   * @summary API ลบข้อมูลวินัย
   * @param disciplineId คีย์วินัย
   */
  @Patch("update-delete/{disciplineId}")
  public async updateIsDeleted(
    @Request() req: RequestWithUser,
    @Path() disciplineId: string,
  ) {
    const record = await this.disciplineRepository.findOneBy({ id: disciplineId });
    if (!record) throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");
    if (record.isDeleted === true) {
      return new HttpSuccess();
    }
    await new permission().PermissionOrgUserDelete(req, "SYS_REGISTRY_EMP", record.profileEmployeeId);
    const before = structuredClone(record);
    const history = new ProfileDisciplineHistory();
    const now = new Date();
    record.isDeleted = true;
    record.lastUpdateUserId = req.user.sub;
    record.lastUpdateFullName = req.user.name;
    record.lastUpdatedAt = now;

    Object.assign(history, { ...record, id: undefined });
    history.profileDisciplineId = disciplineId;
    history.createdUserId = req.user.sub;
    history.createdFullName = req.user.name;
    history.createdAt = now;

    await Promise.all([
      this.disciplineRepository.save(record, { data: req }),
      setLogDataDiff(req, { before, after: record }),
      this.disciplineHistoryRepository.save(history, { data: req }),
    ]);

    return new HttpSuccess();
  }

  @Delete("{disciplineId}")
  public async deleteDiscipline(@Path() disciplineId: string, @Request() req: RequestWithUser) {
    const _record = await this.disciplineRepository.findOneBy({ id: disciplineId });
    if (_record) {
      await new permission().PermissionOrgUserDelete(
        req,
        "SYS_REGISTRY_EMP",
        _record.profileEmployeeId,
      );
    }
    await this.disciplineHistoryRepository.delete({
      profileDisciplineId: disciplineId,
    });

    const result = await this.disciplineRepository.delete({ id: disciplineId });

    if (result.affected == undefined || result.affected <= 0) {
      throw new HttpError(HttpStatus.NOT_FOUND, "ไม่พบข้อมูล");
    }

    return new HttpSuccess();
  }
}
